; note that this code expects the operators
; to be symbols that evaluate to functions,
; not the actual functions

(define (make-exp op left right)
  (list op left right))

(define (operator exp)
  (car exp))

(define (left exp)
  (cadr exp))

(define (right exp)
  (caddr exp))

(define (eval-op op)
  (cond
    ((eq? op '+) +)
    ((eq? op '-) -)
    ((eq? op '/) /)
    ((eq? op '*) *)))

(define (eval-expr exp)
  (if (not (pair? exp))
      exp
      ((eval-op (operator exp))
       (eval-expr (left exp))
       (eval-expr (right exp)))))

(define (evaluator exp)
  (if (not (pair? exp))
      exp
      ((eval (operator exp))
       (evaluator (left exp))
       (evaluator (right exp)))))


(define a '(+ 1 2))                     ; 1+2=3
(define b (make-exp '* a 3))            ; (1+2)*3 = 9
(define c (make-exp '/ a (make-exp '+ 3 4)))  ; (1+2)/(3+4) = 3/7
(define d (make-exp '* c (make-exp '/ 1 c)))  ; c * (1/c) = 1
(define e '(+ 1 (* 2 3)))               ; (1+(2*3)) = 7
(define f '(+ 1 (* 2 (- 3 5))))         ; (1+(2*(3-5))) = -3

a
(eval-expr a)  ; 1+2=3
(evaluator a)
b
(eval-expr b)  ; (1+2)*3 = 9
(evaluator b)
c
(eval-expr c)  ; (1+2)/(3+4) = 3/7
(evaluator c)
d
(eval-expr d)  ; c * (1/c) = 1
(evaluator d)
e
(eval-expr e)  ; (1+(2*3)) = 7
(evaluator e)
f
(eval-expr f)  ; (1+(2*(3-5))) = -3
(evaluator f)
